@@ -16,6 +16,7 @@
 #define NETPBMWRITER_H
 
 #include "poppler-config.h"
+#include "poppler_private_export.h"
 
 #include "ImgWriter.h"
 
@@ -24,29 +25,32 @@
 //   http://netpbm.sourceforge.net/doc/pbm.html
 //   http://netpbm.sourceforge.net/doc/ppm.html
 
-class NetPBMWriter : public ImgWriter
+class POPPLER_PRIVATE_EXPORT NetPBMWriter : public ImgWriter
 {
 public:
+    /* RGB        - 3 bytes/pixel
+     * MONOCHROME - 8 pixels/byte
+     */
+    enum Format
+    {
+        RGB,
+        MONOCHROME
+    };
 
-  /* RGB        - 3 bytes/pixel
-   * MONOCHROME - 8 pixels/byte
-   */
-  enum Format { RGB, MONOCHROME };
+    NetPBMWriter(Format formatA = RGB);
+    ~NetPBMWriter() override {};
 
-  NetPBMWriter(Format formatA = RGB);
-  ~NetPBMWriter() {};
+    bool init(FILE *f, int width, int height, int hDPI, int vDPI) override;
 
-  bool init(FILE *f, int width, int height, int hDPI, int vDPI);
+    bool writePointers(unsigned char **rowPointers, int rowCount) override;
+    bool writeRow(unsigned char **row) override;
 
-  bool writePointers(unsigned char **rowPointers, int rowCount);
-  bool writeRow(unsigned char **row);
-
-  bool close();
+    bool close() override;
 
 private:
-  FILE *file;
-  Format format;
-  int width;
+    FILE *file;
+    Format format;
+    int width;
 };
 
 #endif
