@@ -1,7 +1,7 @@
 /* poppler-page.cc: qt interface to poppler
  * Copyright (C) 2005, Net Integration Technologies, Inc.
  * Copyright (C) 2005, Brad Hards <bradh@frogmouth.net>
- * Copyright (C) 2005-2020, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2005-2021, Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2005, Stefan Kebekus <stefan.kebekus@math.uni-koeln.de>
  * Copyright (C) 2006-2011, Pino Toscano <pino@kde.org>
  * Copyright (C) 2008 Carlos Garcia Campos <carlosgc@gnome.org>
@@ -22,7 +22,7 @@
  * Copyright (C) 2017, 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
  * Copyright (C) 2018 Intevation GmbH <intevation@intevation.de>
  * Copyright (C) 2018, Tobias Deiminger <haxtibal@posteo.de>
- * Copyright (C) 2018 Nelson Benítez León <nbenitezl@gmail.com>
+ * Copyright (C) 2018, 2021 Nelson Benítez León <nbenitezl@gmail.com>
  * Copyright (C) 2020 Oliver Sander <oliver.sander@tu-dresden.de>
  * Copyright (C) 2020 Philipp Knechtges <philipp-dev@knechtges.com>
  *
@@ -50,6 +50,7 @@
 #include <QtGui/QPainter>
 
 #include <config.h>
+#include <poppler-config.h>
 #include <PDFDoc.h>
 #include <Catalog.h>
 #include <Form.h>
@@ -57,7 +58,7 @@
 #include <TextOutputDev.h>
 #include <Annot.h>
 #include <Link.h>
-#include <ArthurOutputDev.h>
+#include <QPainterOutputDev.h>
 #include <Rendition.h>
 #if defined(HAVE_SPLASH)
 #    include <SplashOutputDev.h>
@@ -113,6 +114,8 @@ public:
     {
     }
 
+    ~Qt6SplashOutputDev() override;
+
     void dump() override
     {
         if (partialUpdateCallback && shouldDoPartialUpdateCallback && shouldDoPartialUpdateCallback(payload)) {
@@ -151,7 +154,11 @@ public:
 
             if (takeImageData) {
                 // Construct a Qt image holding (and also owning) the raw bitmap data.
-                return QImage(data, bw, bh, brs, format, gfree, data);
+                QImage i(data, bw, bh, brs, format, gfree, data);
+                if (i.isNull()) {
+                    gfree(data);
+                }
+                return i;
             } else {
                 return QImage(data, bw, bh, brs, format).copy();
             }
@@ -164,10 +171,13 @@ private:
     bool ignorePaperColor;
 };
 
-class QImageDumpingArthurOutputDev : public ArthurOutputDev, public OutputDevCallbackHelper
+Qt6SplashOutputDev::~Qt6SplashOutputDev() = default;
+
+class QImageDumpingQPainterOutputDev : public QPainterOutputDev, public OutputDevCallbackHelper
 {
 public:
-    QImageDumpingArthurOutputDev(QPainter *painter, QImage *i) : ArthurOutputDev(painter), image(i) { }
+    QImageDumpingQPainterOutputDev(QPainter *painter, QImage *i) : QPainterOutputDev(painter), image(i) { }
+    ~QImageDumpingQPainterOutputDev() override;
 
     void dump() override
     {
@@ -180,6 +190,8 @@ private:
     QImage *image;
 };
 
+QImageDumpingQPainterOutputDev::~QImageDumpingQPainterOutputDev() = default;
+
 Link *PageData::convertLinkActionToLink(::LinkAction *a, const QRectF &linkArea)
 {
     return convertLinkActionToLink(a, parentDoc, linkArea);
@@ -417,7 +429,7 @@ static auto shouldAbortExtractionInternalCallback = [](void *user_data) {
 // Needed to make the ternary operator happy.
 static bool (*nullAbortCallBack)(void *user_data) = nullptr;
 
-static bool renderToArthur(QImageDumpingArthurOutputDev *arthur_output, QPainter *painter, PageData *page, double xres, double yres, int x, int y, int w, int h, Page::Rotation rotate, Page::PainterFlags flags)
+static bool renderToQPainter(QImageDumpingQPainterOutputDev *qpainter_output, QPainter *painter, PageData *page, double xres, double yres, int x, int y, int w, int h, Page::Rotation rotate, Page::PainterFlags flags)
 {
     const bool savePainter = !(flags & Page::DontSaveAndRestore);
     if (savePainter)
@@ -428,12 +440,12 @@ static bool renderToArthur(QImageDumpingArthurOutputDev *arthur_output, QPainter
         painter->setRenderHint(QPainter::TextAntialiasing);
     painter->translate(x == -1 ? 0 : -x, y == -1 ? 0 : -y);
 
-    arthur_output->startDoc(page->parentDoc->doc);
+    qpainter_output->startDoc(page->parentDoc->doc);
 
     const bool hideAnnotations = page->parentDoc->m_hints & Document::HideAnnotations;
 
-    OutputDevCallbackHelper *abortHelper = arthur_output;
-    page->parentDoc->doc->displayPageSlice(arthur_output, page->index + 1, xres, yres, (int)rotate * 90, false, true, false, x, y, w, h, abortHelper->shouldAbortRenderCallback ? shouldAbortRenderInternalCallback : nullAbortCallBack,
+    OutputDevCallbackHelper *abortHelper = qpainter_output;
+    page->parentDoc->doc->displayPageSlice(qpainter_output, page->index + 1, xres, yres, (int)rotate * 90, false, true, false, x, y, w, h, abortHelper->shouldAbortRenderCallback ? shouldAbortRenderInternalCallback : nullAbortCallBack,
                                            abortHelper, (hideAnnotations) ? annotDisplayDecideCbk : nullAnnotCallBack, nullptr, true);
     if (savePainter)
         painter->restore();
@@ -533,7 +545,7 @@ QImage Page::renderToImage(double xres, double yres, int xPos, int yPos, int w,
 #endif
         break;
     }
-    case Poppler::Document::ArthurBackend: {
+    case Poppler::Document::QPainterBackend: {
         QSize size = pageSize();
         QImage tmpimg(w == -1 ? qRound(size.width() * xres / 72.0) : w, h == -1 ? qRound(size.height() * yres / 72.0) : h, QImage::Format_ARGB32);
 
@@ -542,16 +554,16 @@ QImage Page::renderToImage(double xres, double yres, int xPos, int yPos, int w,
         tmpimg.fill(bgColor);
 
         QPainter painter(&tmpimg);
-        QImageDumpingArthurOutputDev arthur_output(&painter, &tmpimg);
+        QImageDumpingQPainterOutputDev qpainter_output(&painter, &tmpimg);
 
-        arthur_output.setHintingPreference(QFontHintingFromPopplerHinting(m_page->parentDoc->m_hints));
+        qpainter_output.setHintingPreference(QFontHintingFromPopplerHinting(m_page->parentDoc->m_hints));
 
 #ifdef USE_CMS
-        arthur_output.setDisplayProfile(m_page->parentDoc->m_displayProfile);
+        qpainter_output.setDisplayProfile(m_page->parentDoc->m_displayProfile);
 #endif
 
-        arthur_output.setCallbacks(partialUpdateCallback, shouldDoPartialUpdateCallback, shouldAbortRenderCallback, payload);
-        renderToArthur(&arthur_output, &painter, m_page, xres, yres, xPos, yPos, w, h, rotate, DontSaveAndRestore);
+        qpainter_output.setCallbacks(partialUpdateCallback, shouldDoPartialUpdateCallback, shouldAbortRenderCallback, payload);
+        renderToQPainter(&qpainter_output, &painter, m_page, xres, yres, xPos, yPos, w, h, rotate, DontSaveAndRestore);
         painter.end();
         img = tmpimg;
         break;
@@ -572,12 +584,12 @@ bool Page::renderToPainter(QPainter *painter, double xres, double yres, int x, i
     switch (m_page->parentDoc->m_backend) {
     case Poppler::Document::SplashBackend:
         return false;
-    case Poppler::Document::ArthurBackend: {
-        QImageDumpingArthurOutputDev arthur_output(painter, nullptr);
+    case Poppler::Document::QPainterBackend: {
+        QImageDumpingQPainterOutputDev qpainter_output(painter, nullptr);
 
-        arthur_output.setHintingPreference(QFontHintingFromPopplerHinting(m_page->parentDoc->m_hints));
+        qpainter_output.setHintingPreference(QFontHintingFromPopplerHinting(m_page->parentDoc->m_hints));
 
-        return renderToArthur(&arthur_output, painter, m_page, xres, yres, x, y, w, h, rotate, flags);
+        return renderToQPainter(&qpainter_output, painter, m_page, xres, yres, x, y, w, h, rotate, flags);
     }
     }
     return false;
@@ -826,7 +838,7 @@ QList<FormField *> Page::formFields() const
 {
     QList<FormField *> fields;
     ::Page *p = m_page->page;
-    ::FormPageWidgets *form = p->getFormWidgets();
+    const std::unique_ptr<FormPageWidgets> form = p->getFormWidgets();
     int formcount = form->getNumWidgets();
     for (int i = 0; i < formcount; ++i) {
         ::FormWidget *fm = form->getWidget(i);
@@ -855,8 +867,6 @@ QList<FormField *> Page::formFields() const
             fields.append(ff);
     }
 
-    delete form;
-
     return fields;
 }
 
